#ifndef __ENTITY_CAMERA__
#define __ENTITY_CAMERA__

#include "palmtree.h"
#include "dummy.h"

#define ENTITY_TYPE_CAMERA 5


class EN_Camera : public Entity
{
public:
	GLVECTOR3f posspeed;
	GLVECTOR3f posacc;
	GLVECTOR3f posacc2;
	GLVECTOR3f oldpos;
	GLVECTOR3f targetdiffpos;
	DWORD startpostick;
	DWORD endpostick;

	GLVECTOR3f dirspeed;
	GLVECTOR3f diracc;
	GLVECTOR3f diracc2;
	GLVECTOR3f olddir;
	GLVECTOR3f targetdiffdir;
	GLVECTOR3f targetdir;
	DWORD startdirtick;
	DWORD enddirtick;

	GLVECTOR3f postrackoffset;
	GLVECTOR3f *postrack;
	GLVECTOR3f *dirtrack;

	GLVECTOR3f *looktarget;

	int trackdelay;
	DWORD nexttrack;


	void findnextpalmtree();

	void setpostarget(GLVECTOR3f *p,int delay);
	void setdirtarget(GLVECTOR3f *p, int delay);
	void trackpos(GLVECTOR3f *e) {postrack = e;};
	void trackdir(GLVECTOR3f *e) {dirtrack = e;looktarget = NULL;};
	void settrackdelay(int delay) {trackdelay = delay;};
	void lookat(GLVECTOR3f *p);

	int renderme() {return(1);};
	int moveme();
	int init();
	int beforesave() {return(1);}; //Do preparing stuff before saving. (Fill the savebuffer, for example?)
	int afterload() {return(1);};  //Do initializing stuff after loading. Will be called AFTER init();
	
	EN_Camera();
};


void EN_Camera::lookat(GLVECTOR3f *p)
{
	looktarget = p;
	if (!p) return;
	static GLVECTOR3f temp;
	temp = pos - *p;
	temp.normalize();
	temp *= -1;
	currentworld->v2a(&temp,&dir);
}


int EN_Camera::init()
{
	currentworld->setcameraref(&pos); /* Take control of the world's eye */
	currentworld->setcameradirref(&dir);
	return(1);
}

EN_Camera::EN_Camera()
{
	looktarget = NULL;
	type = ENTITY_TYPE_CAMERA; 
	homesector = ENTITY_STATIC;
	endpostick = 0;
	enddirtick = 0;
	postrack = NULL;
	dirtrack = NULL;
	nexttrack = 0;
	trackdelay = 100;
}

void EN_Camera::findnextpalmtree()
{
	static int index = 0;
	int startindex;
	startindex = index;
	index = (index+1)%currentworld->entities;

	while (currentworld->entity[index]->type != ENTITY_TYPE_PALMTREE && index != startindex)
	{
		index = (index+1)%currentworld->entities;	
	}
	setpostarget(&currentworld->entity[index]->pos,5000);	
	GLVECTOR3f d;
	d = currentworld->entity[index]->pos - pos;
	d.normalize();
	currentworld->v2a(&d);

//	setdirtarget(&currentworld->entity[index]->dir,2000);
	setdirtarget(&d,2000);
}


int EN_Camera::moveme()
{
	static GLVECTOR3f add;
	static GLVECTOR3f add2;
	static GLVECTOR3f temp;
	static float elaps;
	static float height;



	if (currentworld->worldTickCount >= nexttrack)
	{
		if (postrack) 
		{
			temp = *postrack;
			
			currentworld->a2v(&targetdir,&add);
			temp += add*5.0f;
			add2 = add;
			if (add.y == 0)
			{
				add.set(0,1,0);				
			}
			else if (add.y > 0)
			{
				add2 *= -1.0f;
				add2.y = -(add.x*add2.x+add.z*add2.z)/add.y;
				add2.normalize();
			}
			else 
			{
				add2.y = -(add.x*add2.x+add.z*add2.z)/add.y;
				add2.normalize();
			}
			temp += add2*3.0f;

			setpostarget(&temp,300);
		}
		if (dirtrack)
		{
			setdirtarget(dirtrack,300);
		}
		nexttrack = currentworld->worldTickCount + trackdelay;
	}



	if (currentworld->worldTickCount > endpostick)
	{
//		posspeed.set(0,0,0);
		pos = oldpos + targetdiffpos;
	}
	else
	{
		elaps = (float)(currentworld->worldTickCount-startpostick)/(float)(endpostick-startpostick);
		add = posacc2*pow(elaps,3) + posacc*pow(elaps,2) + posspeed*elaps;
		pos = oldpos + add;

	}

	if (currentworld->worldTickCount > enddirtick)
	{
		dir = olddir + targetdiffdir;
	}
	else
	{
		elaps = (float)(currentworld->worldTickCount-startdirtick)/(float)(enddirtick-startdirtick);
		add = diracc2*pow(elaps,3) + diracc*pow(elaps,2) + dirspeed*elaps;
		dir = olddir + add;
	}

	height = currentworld->getheight(pos.x,pos.z);
	if (pos.y < height + 1) pos.y = height + 1; 

	if (keys[SDLK_g]) 
	{
		keys[SDLK_g] = 0;
		findnextpalmtree();
	}

	if (looktarget)
		lookat(looktarget);


	return(1);
}



void EN_Camera::setdirtarget(GLVECTOR3f *p, int delay)
{
	looktarget = NULL;
	static GLVECTOR3f temp;
	dir.x = fmod(dir.x,360);
	if (dir.x < 0) dir.x = 360+dir.x;
	dir.y = fmod(dir.y,360);
	if (dir.y < 0) dir.y = 360+dir.y;

	temp = *p;
	
	temp.x = fmod(temp.x,360);
	if (temp.x < 0) temp.x = 360+temp.x;
	temp.y = fmod(temp.y,360);
	if (temp.y < 0) temp.y = 360+temp.y;


	if (fabs(temp.x-dir.x) < fabs(temp.x > dir.x? 360-temp.x + dir.x : 360-dir.x + temp.x ) )
		targetdiffdir.x = temp.x-dir.x;
	else
	{
		if (temp.x > dir.x)
			targetdiffdir.x = -dir.x - 360 + temp.x;
		else
		targetdiffdir.x = 360-dir.x + temp.x;
	}

	

	if (fabs(temp.y-dir.y) < fabs((temp.y > dir.y? 360-temp.y + dir.y : 360-dir.y + temp.y) ) )
		targetdiffdir.y = temp.y-dir.y;
	else
	{
		if (temp.y > dir.y)
			targetdiffdir.y = -dir.y - 360 + temp.y;
		else
		targetdiffdir.y = 360-dir.y + temp.y;
	}

	

	targetdiffdir.z = 0;

	olddir = dir;
	targetdir = olddir + targetdiffdir;

	float elaps;
	if (currentworld->worldTickCount > enddirtick)
	{
		dirspeed.set(0,0,0);
	}
	else
	{
		elaps = (float)(currentworld->worldTickCount-startdirtick)/(float)(enddirtick-startdirtick);
		dirspeed += diracc2*3*pow(elaps,2) + diracc*2*elaps;
	}
	

	diracc = dirspeed*-2.0f;
	diracc += targetdiffdir*3.0f;
	diracc2 = dirspeed;
	diracc2 += targetdiffdir*-2.0f;

	startdirtick = currentworld->worldTickCount;
	enddirtick = startdirtick + delay;
}



void EN_Camera::setpostarget(GLVECTOR3f *p, int delay)
{
	targetdiffpos = *p-pos;
	targetdiffpos.x += 1;
	oldpos = pos;

	float elaps;
	if (currentworld->worldTickCount > endpostick)
	{
		posspeed.set(0,0,0);
	}
	else
	{
		elaps = (float)(currentworld->worldTickCount-startpostick)/(float)(endpostick-startpostick);
		posspeed += posacc2*3*pow(elaps,2) + posacc*2*elaps;
	}
	

	posacc = posspeed*-2.0f;
	posacc += targetdiffpos*3.0f;
	posacc2 = posspeed;
	posacc2 += targetdiffpos*-2.0f;

	startpostick = currentworld->worldTickCount;
	endpostick = startpostick + delay;
}


#endif