#ifndef __ENTITY_EXPLOSION__
#define __ENTITY_EXPLOSION__

#define EN_Exp_Particles 30

class EN_Explosion : public Entity
{
public:
	GLVECTOR3f particle[EN_Exp_Particles];
	GLVECTOR3f particlespeed[EN_Exp_Particles];
	int isalive[EN_Exp_Particles];
	GLuint texture;
	int flarecount;
	virtual int renderme();
	virtual int moveme();
	virtual int init();
	virtual int beforesave() {return(1);}; //Do preparing stuff before saving. (Fill the savebuffer, for example?)
	virtual int afterload() {return(1);};  //Do initializing stuff after loading. Will be called AFTER init();

};


int EN_Explosion::moveme()
{
	static int num;
	static int someisalive;

	dir = *currentworld->cameradir;

	someisalive = 0;
	for (num = 0; num < EN_Exp_Particles; num++)
	if (isalive[num])
	{
		someisalive = 1;
  		particle[num] += particlespeed[num];
		particlespeed[num].y -= 0.01; //gravity
		if (currentworld->getheight(particle[num].x+pos.x,particle[num].z+pos.z) > particle[num].y+pos.y || particle[num].y+pos.y < currentworld->waterlevel)
			isalive[num] = 0;
	}
	if (!someisalive) REMOVEME = 1; //Nothing left to render
	return(1);
}

int EN_Explosion::renderme()
{
	static int num;
	static GLVECTOR3f corner[4] = {GLVECTOR3f(-0.5,0.5,0),GLVECTOR3f(0.5,0.5,0),GLVECTOR3f(0.5,-0.5,0),GLVECTOR3f(-0.5,-0.5,0)};
	static GLVECTOR3f temp;
	glColor3f(1,0.4,0);
	glEnable(GL_BLEND);
	glDepthMask(GL_FALSE);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glDisable(GL_FOG);
	glBindTexture(GL_TEXTURE_2D,texture);
//	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
		for (num = 0; num < EN_Exp_Particles; num++)
			if (isalive[num])
			{
				glTexCoord2f(0,0);
				temp = particle[num] + corner[0];
				glVertex3fv((float*)&temp);
				glTexCoord2f(1,0);
				temp = particle[num] + corner[1];
				glVertex3fv((float*)&temp);
				glTexCoord2f(1,1);
				temp = particle[num] + corner[2];
				glVertex3fv((float*)&temp);
				glTexCoord2f(0,1);
				temp = particle[num] + corner[3];
				glVertex3fv((float*)&temp);
			}
		if (flarecount)
		{
			glColor3f(1,1,1);
			glTexCoord2f(0,0);
			glVertex3f(-3,-3,0);
			glTexCoord2f(1,0);
			glVertex3f(3,-3,0);
			glTexCoord2f(1,1);
			glVertex3f(3,3,0);
			glTexCoord2f(0,1);
			glVertex3f(-3,3,0);
			flarecount--;
		}
	glEnd();
	glDisable(GL_BLEND);
	glDepthMask(GL_TRUE);
	glEnable(GL_FOG);
	return(1);
}



int EN_Explosion::init()
{
	dir.set(0,0,0);
	homesector = ENTITY_EFFECT;
	int num;
	static GLVECTOR3f *n;
	static Sector *s;
	static int xl,yl;

	flarecount = 20;
	texture = currentworld->texturehandler.gettexture(currentworld->texturehandler.load("data\\textures\\alphaball.mpc",2));
	xl = pos.x/currentworld->xlen;
	yl = pos.y/currentworld->ylen;
	s = &currentworld->sector[xl+yl*currentworld->amx];
	n = &s->normal[(int)(pos.x + xl*currentworld->xlen) + (int)(pos.y + yl*currentworld->ylen)*s->amx];
	for (num = 0; num < EN_Exp_Particles; num++)
	{
		particle[num].set(0,0.1,0);
//		particlespeed[num] = *n;
//		particlespeed[num] *= -1;
		particlespeed[num].x += (float)((rand()%100)-50)/500.0f;
		particlespeed[num].y += (float)((rand()%100)-50)/500.0f;
		particlespeed[num].z += (float)((rand()%100)-50)/500.0f;
		particlespeed[num].normalize();
		particlespeed[num] *= 0.3;
		isalive[num] = 1;
	}
	return(1);
}



#endif