#ifndef __GLMATRIX3F__
#define __GLMATRIX3F__


#include "mystuff/glvector3f.h"
#include "mystuff/debug.h"

/*
012
345
678
*/


#define MATHRES 0.00001
//#define ISZERO(val) fabs(val) < MATHRES
//#define ISOVERZERO(val) fabs(val) >= MATHRES

#define ISZERO(val) !val
#define ISOVERZERO(val) val


struct GLMATRIX3f
{
	float value[9];

	void operator=(GLMATRIX3f other);
	void setrow(int index,GLVECTOR3f row);
	void setcolumn(int index,GLVECTOR3f column);
	GLVECTOR3f getrow(int index);
	GLVECTOR3f getcolumn(int index);
	GLMATRIX3f();
	GLMATRIX3f operator*(GLMATRIX3f other);
	GLVECTOR3f operator*(GLVECTOR3f other);
	void operator*=(GLMATRIX3f other);

	int countvaluesinrow(int index);
	void transponate();
	int invert();
	int findrow(int notrow,int targetindex,int reservedindex);
	int getindexinrow(int index,int withvalue);
	void put(DebugClass *d);
	void set(float *val) {memcpy(value,val,sizeof(float)*9);};
	void makegoodzeros();
};

void GLMATRIX3f::makegoodzeros()
{
	int num;
	for (num = 0; num < 9; num++)
	{
		if (fabs(value[num]) < MATHRES) value[num] = 0;
	}
}


int GLMATRIX3f::getindexinrow(int index,int withvalue)
{
	int num;
	for (num = 0; num < 3; num++)
	{
		if (withvalue)
		{
			if (ISOVERZERO(value[num+index*3])) return(num);
		}
		else
		{
			if (ISZERO(value[num+index*3])) return(num);
		}
	}
	return(-1);
}




void GLMATRIX3f::put(DebugClass *d)
{
	d->put("Matrix put:\n");
	int row,num;
	for (row = 0; row < 3; row++)
	{
		for (num = 0; num < 3; num++)
		{
			d->put(value[num+row*3]);
			if (num != 2)
				d->put(",");
		}
		d->newline();
	}
	
}



int GLMATRIX3f::findrow(int notrow,int targetindex,int reservedindex)
{
	int num;
	for (num = 0; num < 3; num++)
	{
		if (notrow != -1)
			if (num == notrow) continue;
		if (targetindex != -1)
			if (ISZERO(value[num*3+targetindex])) continue;

		if (reservedindex != -1)
			if (ISOVERZERO(value[num*3+reservedindex])) continue;

		return(num);
	}
	return(-1);
}


int GLMATRIX3f::countvaluesinrow(int index)
{
	int count = 0;
	int num;
	for (num = 0; num < 3; num++)
		if (ISOVERZERO(value[num+index*3])) count++;

	return(count);
}


/*
	Functions needed:
	int countvaluesinrow(int index); //Retuns how many elements in the row that has a non-zero value

	//Returns a row that meet the specified requirement, or -1 if not found.
	int findrow(int notrow,int targetindex,int reservedindex);

	//Returns the first element's index in a row that has the value 0.
	int getindexinrow(int row,int withvalue);
*/


int GLMATRIX3f::invert()
{
	GLMATRIX3f inverted;
	GLVECTOR3f tempv,tempv2;

	int count = 100;
	int trow,norow;
	int row,num;
	int rowswithonevalue;
	float base;
	int insertto[3];

	while(count--)
	{
		
		rowswithonevalue = 0;

		for (row = 0; row < 3; row++)
		{
			switch(countvaluesinrow(row))
			{
			case 0:
				return(0);
			break;
			case 1:
				rowswithonevalue++;
			break;
			case 2:
				norow = getindexinrow(row,0);
				for (num = 0; num < 3; num++)
				{
					trow = findrow(row,num,norow);
					if (trow != -1) break;
				}
				if (trow == -1) continue; //Not a good time to do this now.
				base = value[num+row*3]/value[num+trow*3];	

				tempv.set(0,0,0);
				tempv -= getrow(trow);
				tempv *= base;
				tempv += getrow(row);
				tempv.setelement(num,0);
				setrow(row,tempv);

				tempv.set(0,0,0);
				tempv -= inverted.getrow(trow);
				tempv *= base;
				tempv += inverted.getrow(row);

//				tempv.setelement(num,0);
				inverted.setrow(row,tempv);

			break;
			case 3:
				for (num = 0; num < 3; num++)
				{
					trow = findrow(row,num,-1);
					if (trow != -1) break;
				}
				if (trow == -1) return(0); //This should NEVER happen!
				base = value[num+row*3]/value[num+trow*3];	

				tempv.set(0,0,0);
				tempv -= getrow(trow);
				tempv *= base;
				tempv += getrow(row);

				tempv.setelement(num,0);

				setrow(row,tempv);

				tempv.set(0,0,0);
				tempv -= inverted.getrow(trow);
				tempv *= base;
				tempv += inverted.getrow(row);

//				tempv.setelement(num,0);

				inverted.setrow(row,tempv);


			break;


			}
		

		}

		if (rowswithonevalue == 3)
		{
			//This is where the end process goes!
			for (row = 0; row < 3; row++)
			{
				num = getindexinrow(row,1);
				insertto[row] = num;
				if (num == -1) return(0);

				base = 1.0f/value[num+row*3];
				tempv = inverted.getrow(row);
				tempv *= base;
				inverted.setrow(row,tempv);
			}
			
			for (num = 0; num < 3; num++)
			{
				setrow(insertto[num],inverted.getrow(num));
			}

//			makegoodzeros();
			return(1);

		}
		
	}

	return(0);
}




GLVECTOR3f GLMATRIX3f::getrow(int index)
{
	return(GLVECTOR3f(value[index*3],value[index*3+1],value[index*3+2]));
}

GLVECTOR3f GLMATRIX3f::getcolumn(int index)
{
	return(GLVECTOR3f(value[index],value[index+3],value[index+6]));
}


void GLMATRIX3f::transponate()
{
	GLMATRIX3f src = *this;
	int xnum,ynum;
	for (ynum = 0; ynum < 3; ynum++)
		for (xnum = 0; xnum < 3; xnum++)
		{
			value[xnum+ynum*3] = src.value[ynum+xnum*3];
		}
}


GLMATRIX3f GLMATRIX3f::operator*(GLMATRIX3f other)
{
	GLMATRIX3f m = *this;
	m*=other;
	return(m);
}


GLVECTOR3f GLMATRIX3f::operator*(GLVECTOR3f other)
{
	static GLVECTOR3f temp;
	static GLVECTOR3f ret;
	static int num;

		temp = other*getrow(0);
		ret.x = temp.x+temp.y+temp.z;		
		temp = other*getrow(1);
		ret.y = temp.x+temp.y+temp.z;		
		temp = other*getrow(2);
		ret.z = temp.x+temp.y+temp.z;		

	return(ret);
}


void GLMATRIX3f::operator*=(GLMATRIX3f other)
{
	GLVECTOR3f vec;

	GLMATRIX3f m = *this;
	int xnum,ynum;
	float ret;

	for (xnum = 0; xnum < 3; xnum++)
		for (ynum = 0; ynum < 3; ynum++)
		{
			vec = m.getrow(ynum) * other.getcolumn(xnum);

			ret = vec.x + vec.y + vec.z;
			value[xnum+ynum*3] = ret;
		}

	
}






GLMATRIX3f::GLMATRIX3f()
{
	value[0] = 1;
	value[1] = 0;
	value[2] = 0;
	value[3] = 0;
	value[4] = 1;
	value[5] = 0;
	value[6] = 0;
	value[7] = 0;
	value[8] = 1;
}


void GLMATRIX3f::operator=(GLMATRIX3f other)
{
	memcpy(value,other.value,sizeof(float)*9);
}

void GLMATRIX3f::setrow(int index,GLVECTOR3f row)
{
	value[3*index] = row.x;
	value[3*index+1] = row.y;
	value[3*index+2] = row.z;
}

void GLMATRIX3f::setcolumn(int index,GLVECTOR3f column)
{
	value[index] = column.x;
	value[index+3] = column.y;
	value[index+6] = column.z;
}


#endif