#ifndef __GLVECTOR3f__
#define __GLVECTOR3f__


#include "mystuff/GLVECTOR2F.h"
#define GLVECTOR3F_ACCURATE_COMPARE

#define GLVECTOR3f_COMPARE_THRES 0.001


/*
	Handles a 3-dimensional vector.

*/

struct GLVECTOR3f
{
	float x,y,z;
	void set(float _x = 0,float _y = 0 ,float _z = 0);
	void operator+=(GLVECTOR3f other);
	void operator-=(GLVECTOR3f other);
	void operator*=(float value) {x *= value; y *= value; z *= value;};
	void operator*=(GLVECTOR3f v) {x *= v.x; y *= v.y; z *= v.z;};
	GLVECTOR3f operator*(GLVECTOR3f other) {return(GLVECTOR3f(other.x*x,other.y*y,other.z*z));};
	int operator==(GLVECTOR3f other);
	GLVECTOR3f operator+(GLVECTOR3f other);
	GLVECTOR3f operator-(GLVECTOR3f other);
	GLVECTOR3f(float _x,float _y,float _z) {x = _x;y = _y;z = _z;};
	GLVECTOR3f operator * (float value);
	void operator=(GLVECTOR3f other);
	void operator=(GLVECTOR2f other);
	GLVECTOR3f() {x = 0; y = 0; z = 0;};
	float len();
	float lenfrom(GLVECTOR3f *other);
	void normalize();
	float getelement(int index);
	void setelement(int index,float value);
};



void GLVECTOR3f::setelement(int index,float value)
{
	switch(index)
	{
	case 0:
		x = value;
	break;
	case 1:
		y = value;
	break;
	case 2:
		z = value;
	break;
	}
}


float GLVECTOR3f::getelement(int index)
{
	switch(index)
	{
	case 0:
		return(x);
	break;
	case 1:
		return(y);
	break;
	case 2:
		return(z);
	break;
	default:
		return(-1);
	}
}

void GLVECTOR3f::normalize()
{
	float invlen;
	float l;
	l = len();
	if (l == 0)
	{
		set(0,1,0);
		return;
	}

	invlen = 1/l;
	x *= invlen;
	y *= invlen;
	z *= invlen;
}

void GLVECTOR3f::set(float _x,float _y,float _z)
{
	x = _x;
	y = _y;
	z = _z;
}

float GLVECTOR3f::len()
{
	return(sqrt(x*x+y*y+z*z));
}

GLVECTOR3f GLVECTOR3f::operator-(GLVECTOR3f other)
{
	return(GLVECTOR3f(x-other.x,y-other.y,z-other.z));	
}

void GLVECTOR3f::operator=(GLVECTOR3f other)
{
	x = other.x;
	y = other.y;
	z = other.z;
}
void GLVECTOR3f::operator=(GLVECTOR2f other)
{
	x = other.x;
	y = other.y;
	z = 0;
}

#ifdef GLVECTOR3f_ACCURATE_COMPARE


int GLVECTOR3f::operator==(GLVECTOR3f other)
{
	return((x==other.x) && (y==other.y) && (z == other.z));
}

#else

int GLVECTOR3f::operator==(GLVECTOR3f other)
{
	return((fabs(x-other.x) < GLVECTOR3f_COMPARE_THRES) && (fabs(y-other.y) < GLVECTOR3f_COMPARE_THRES) && (fabs(z < other.z) < GLVECTOR3f_COMPARE_THRES));
}

#endif


GLVECTOR3f GLVECTOR3f::operator+(GLVECTOR3f other)
{
	return(GLVECTOR3f(x+other.x,y+other.y,z+other.z));
}

GLVECTOR3f GLVECTOR3f::operator*(float value)
{
	return(GLVECTOR3f(x*value,y*value,z*value));
}

void GLVECTOR3f::operator +=(GLVECTOR3f other)
{
	x += other.x;
	y += other.y;
	z += other.z;
//	return
}

void GLVECTOR3f::operator -=(GLVECTOR3f other)
{
	x -= other.x;
	y -= other.y;
	z -= other.z;
}




float GLVECTOR3f::lenfrom(GLVECTOR3f *other)
{
	GLVECTOR3f ret(x,y,z);
	ret -= *other;
	ret *= ret;
	return(sqrt(ret.x+ret.y+ret.z));
}



#endif