#ifndef __RAYTRACER__
#define __RAYTRACER__

#include "world.h"

class RayTracer
{
public:
	World *world;
	GLVECTOR3f sundir;
	GLVECTOR3f shadowmod;
	GLVECTOR3f ambientcolor;
	GLVECTOR3f suncolor;

	void settarget(World *t) {world = t;};
	void setsundir(GLVECTOR3f dir) {sundir = dir;sundir.normalize(); if(world) world->sundir = sundir;};
	void multsunintensity(float mult) {sundir *= mult;};
	void setshadowmod(GLVECTOR3f sh) {shadowmod = sh;};
	void traceshadows(int showstatusbar = 0);
	void setambient();
	void makesunlight();
	RayTracer();
};


void RayTracer::setambient()
{
	int sindex,num;
	for (sindex = 0; sindex < world->sectors; sindex++)
	{
		for (num = 0; num < world->sector[sindex].faces; num++)
		{
			world->sector[sindex].face[num].color[0] = ambientcolor;
			world->sector[sindex].face[num].color[1] = ambientcolor;
			world->sector[sindex].face[num].color[2] = ambientcolor;
			world->sector[sindex].face[num].color2[0].set(ambientcolor.x,ambientcolor.y,ambientcolor.z);
			world->sector[sindex].face[num].color2[1].set(ambientcolor.x,ambientcolor.y,ambientcolor.z);
			world->sector[sindex].face[num].color2[2].set(ambientcolor.x,ambientcolor.y,ambientcolor.z);
		}
	}
}


void RayTracer::makesunlight()
{
	if (!world) return;

	int sindex;
	int num;
	Sector *s;
	GLVECTOR3f dir;
	GLVECTOR3f dir2;
	GLVECTOR4f col4;
	float len;

	for (sindex = 0; sindex < world->sectors; sindex++)
	{
		s = &world->sector[sindex];
		for (num = 0; num < s->vertexes; num++)
		{			
//			if (s->color[num].x < ambientcolor.x) continue; //Shadowed

			dir = s->normal[num] + sundir;
			dir2 = dir * s->normal[num]; //Skalrprodukt
			len = (dir2.x+dir2.y+dir2.z);
			len /= 2.0f;			

//			if (len < 1) len = 1;

/*			s->color[num] += suncolor*len;
			s->color2[num].x += suncolor.x*len;
			s->color2[num].y += suncolor.y*len;
			s->color2[num].z += suncolor.z*len;
*/
			col4.set(suncolor.x*len,suncolor.y*len,suncolor.z*len,0);

			s->setcolor(num,0,suncolor*len,1);
			s->setcolor2(num,0,col4,1);

/*
			if (s->color[num].x < ambientcolor.x) s->color[num].x = ambientcolor.x;
			if (s->color[num].y < ambientcolor.y) s->color[num].y = ambientcolor.y;
			if (s->color[num].z < ambientcolor.z) s->color[num].z = ambientcolor.z;
			if (s->color2[num].x < ambientcolor.x) s->color2[num].x = ambientcolor.x;
			if (s->color2[num].y < ambientcolor.y) s->color2[num].y = ambientcolor.y;
			if (s->color2[num].z < ambientcolor.z) s->color2[num].z = ambientcolor.z;
*/

		}
	}

}








void RayTracer::traceshadows(int showstatusbar)
{
	if (!world) return;

	world->nudgeedges(100);
	world->finalize();
	
	if (showstatusbar)
	{
		glPushMatrix();
		glLoadIdentity();
		glTranslatef(0,0,-5);
	}


	int sindex,sindex2; 
	int num,num2,num3;
	int shadowed;
	int indice[1024];
	int count;
	Sector *s;
	Sector *s2;

	for (sindex = 0; sindex < world->sectors; sindex++)
	{


		s = &world->sector[sindex];

		if (showstatusbar)
		{
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glColor3f(1,1,1);
			glBegin(GL_LINE_STRIP);
				glVertex3f(0,0,0);
				glVertex3f(3,0,0);
				glVertex3f(3,0.5,0);
				glVertex3f(0,0.5,0);
				glVertex3f(0,0,0);
			glEnd();
			glColor3f(0.3,0.3,1);
			glBegin(GL_QUADS);
				glVertex3f(0,0,0);
				glVertex3f(((float)sindex/(float)world->sectors)*3.0,0,0);
				glVertex3f(((float)sindex/(float)world->sectors)*3.0,0.5,0);
				glVertex3f(0,0.5,0);
			glEnd();
			SDL_GL_SwapBuffers();
		}


		for (num = 0; num < s->vertexes; num++)
		{



			shadowed = 0;
			for (sindex2 = 0; sindex2 < world->sectors; sindex2++)
			{
				s2 = &world->sector[sindex2];
				if (!s2->intersectingarea(s->vertex[num],sundir)) continue;

				s2->getcrossingfaces(s->vertex[num],sundir,indice,1024,&count);

				for (num2 = 0; num2 < count; num2++)
				{
					if ((s->vertex[num] == s2->vertex[s2->face[indice[num2]].vertice[0]]) ||
					    (s->vertex[num] == s2->vertex[s2->face[indice[num2]].vertice[1]]) ||
					    (s->vertex[num] == s2->vertex[s2->face[indice[num2]].vertice[2]]))
						continue;

					if (s2->intersecting(indice[num2],s->vertex[num],sundir))
					{
						shadowed = 1;
						sindex2 = world->sectors;
						break;
					}



				}



			}

			if (shadowed)
			{
//				s->color[num] *= shadowmod;
				s->setcolor(num,0,shadowmod,3);
				s->setcolor2(num,0,GLVECTOR4f(shadowmod.x,shadowmod.y,shadowmod.z,1),3);
//				s->color2[num].x *= shadowmod.x;
//				s->color2[num].y *= shadowmod.y;
//				s->color2[num].z *= shadowmod.z;
			}




		}



	}


	if (showstatusbar)
	{
		glPopMatrix();
	}
	world->nudgeedges(-100);

}





RayTracer::RayTracer()
{
	world = NULL;
	suncolor.set(1,1,1);
	ambientcolor.set(0.5,0.5,0.5);
}




#endif