#ifndef __SECTOR__
#define __SECTOR__

#include "mystuff/plane.h"
#include "mystuff/glvector2f.h"
#include "mystuff/glvector4f.h"
#include "mystuff/glvector3f.h"
#include "mystuff/glmatrix3f.h"
#include "mystuff/mymath.h"
#include "entity.h"



#define S_MAXENTITIES 100



#define SAREA_LEFT 0
#define SAREA_TOP 1
#define SAREA_RIGHT 2
#define SAREA_BOTTOM 3

class SectorArea
{
	float area[4]; // left, top, right, bottom
	float maxheight;
	float minheight;


public:
	void setmaxheight(float h) {maxheight = h;};
	float getmaxheight() {return(maxheight);};
	void setminheight(float h) {minheight = h;};
	float getminheight() {return(minheight);};
	void setarea(float left, float top, float right, float bottom) {area[SAREA_LEFT] = left; area[SAREA_TOP] = top; area[SAREA_RIGHT] = right; area[SAREA_BOTTOM] = bottom;};
	float getpos(int side) {return(area[side]);};
	int inarea(float x,float y) {return((x >= area[SAREA_LEFT] && x <= area[SAREA_RIGHT] && y >= area[SAREA_TOP] && y <= area[SAREA_BOTTOM]));};
	float getlen(float x,float y,float z);
	int isinfront(GLVECTOR3f *v1,GLVECTOR3f *v2);
	SectorArea() {maxheight = 0;minheight = 0;};

	int intersectingarea(GLVECTOR3f pos,GLVECTOR3f dir);
};


int SectorArea::intersectingarea(GLVECTOR3f pos,GLVECTOR3f dir)
{

	if (inarea(pos.x,pos.z)) return(1);

	GLVECTOR2f line1,line2;
	GLVECTOR2f sline1,sline2;


	sline1.set(pos.x,pos.z);
	sline2.set(dir.x,dir.z);
	dir *= 1000;
	sline2 *= 1000;
	sline2 += sline1;
	float len,len2;




	//Check top
	line1.set(area[SAREA_LEFT],area[SAREA_TOP]);
	line2.set(area[SAREA_RIGHT],area[SAREA_TOP]);
	len = crossingline(&sline1,&sline2,&line1,&line2,&len2);
	if (len >= 0)
	{
		return(1);
	}



	//Check right
	line1.set(area[SAREA_RIGHT],area[SAREA_TOP]);
	line2.set(area[SAREA_RIGHT],area[SAREA_BOTTOM]);
	len = crossingline(&sline1,&sline2,&line1,&line2,&len2);
	if (len >= 0)
	{
		return(1);
	}



	//Check bottom
	line1.set(area[SAREA_RIGHT],area[SAREA_BOTTOM]);
	line2.set(area[SAREA_LEFT],area[SAREA_BOTTOM]);
	len = crossingline(&sline1,&sline2,&line1,&line2,&len2);
	if (len >= 0)
	{
		return(1);
	}



	//Check left
	line1.set(area[SAREA_LEFT],area[SAREA_BOTTOM]);
	line2.set(area[SAREA_LEFT],area[SAREA_TOP]);
	len = crossingline(&sline1,&sline2,&line1,&line2,&len2);
	if (len >= 0)
	{
		return(1);
	}



	return(0);
}


int SectorArea::isinfront(GLVECTOR3f *v1,GLVECTOR3f *v2)
{
	static GLVECTOR3f pos;

	pos.set(area[SAREA_LEFT],maxheight,area[SAREA_TOP]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);
	pos.set(area[SAREA_LEFT],minheight,area[SAREA_TOP]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);

	pos.set(area[SAREA_LEFT],maxheight,area[SAREA_BOTTOM]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);
	pos.set(area[SAREA_LEFT],minheight,area[SAREA_BOTTOM]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);

	pos.set(area[SAREA_RIGHT],maxheight,area[SAREA_TOP]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);
	pos.set(area[SAREA_RIGHT],minheight,area[SAREA_TOP]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);

	pos.set(area[SAREA_RIGHT],maxheight,area[SAREA_BOTTOM]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);
	pos.set(area[SAREA_RIGHT],minheight,area[SAREA_BOTTOM]);
	if (pos.lenfrom(v2) >= pos.lenfrom(v1)) return(1);


	return(0);

}


float SectorArea::getlen(float x,float y,float z)
{
	static GLVECTOR3f v;
	v.set(area[SAREA_LEFT] + (area[SAREA_RIGHT] - area[SAREA_LEFT] - 0.1)/2.0f,
		  0,
		  area[SAREA_TOP] + (area[SAREA_BOTTOM] - area[SAREA_TOP] - 0.1)/2.0f);
	v -= GLVECTOR3f(x,0,z);
	return(v.len());
}





///////////////////////////////////////////////////////


/*
   Face 1                Face 2
		0_______1    1
		|       /   /|
        |      /   / |
        |     /   /  |
        |    /   /   |
        |   /   /    |
        |  /   /     |
        | /   /      |
        2/   2-------0

*/




struct SectorFace
{
//	GLVECTOR2f texcoord[3];
//	GLVECTOR2f texcoord2[3];
	GLVECTOR3f color[3];
	GLVECTOR4f color2[3];
	GLuint texture;
	GLuint texture2;
	int vertice[3];
	GLMATRIX3f planematrix;
	int intersecting(GLVECTOR3f pos,GLVECTOR3f dir);
};



int SectorFace::intersecting(GLVECTOR3f pos,GLVECTOR3f dir)
{
	pos = planematrix*pos;
	dir = planematrix*dir;
	pos = pos + dir*(-pos.z/dir.z);
	if (pos.x >= 0 && pos.x <= 1 && pos.y >= 0 && pos.y <= 1 && pos.x + pos.y <= 1) 
	return(1); else return(0);
}



struct SectorFileHeader
{
	WORD vertexes;
	WORD faces;
	float xlen,ylen;
	WORD amx,amy;
};



class Sector : public SectorArea
{
private:
	void calcbump(int x,int y,float *map,int mapx,int mapy,int mapxlen,int mapylen);

public:
	GLVECTOR3f *vertex; //Is the sector's physical data
	GLVECTOR3f *normal;
//	GLVECTOR3f *color;
//	GLVECTOR4f *color2;
	GLVECTOR2f *texcoord;
	GLVECTOR2f *texcoord2;
	int vertexes; //How many objects in vertex,normal,color and color2
	float xlen,ylen; //The length between each vertex
	int amx,amy; //The grid size of vertexes. (vertexes = amx*amy)

	SectorFace *face; //Is the array of the faces on the sector.
	int faces; //Specifies the amount of faces.


	int *faceindex1; //Specifies the face indices, to use for optimal first render
	int *faceindex2; //Specifies the face indices, to use for optimal second render.


	//read-only functions:
	float getheight(float x,float y); //Gets the height of the plane on the specified position, or -999.9 if not in area.
	float getdistance(GLVECTOR3f pos,GLVECTOR3f dir);
	void getcrossingfaces(GLVECTOR3f pos,GLVECTOR3f dir,int *indice,int max,int *count); //Returns all the faces in the sector that is crossing the specified ray


	//system functions:
	void clear(); //Removes all dynamically stored data in the object.

	
	//Manipulative functions:
	void setcolor(int x,int y,GLVECTOR3f col,int action = 0);
	void setcolor2(int x,int y,GLVECTOR4f col,int action = 0);
	void fillcolor(GLVECTOR3f col); //Specifies a global color to set on all vertexes.
	void fillcolor2(GLVECTOR4f col); //Specifies a global second color to set on all vertexes.
	void filltexture(GLuint texture); //Specifies a global texture indice to set on all faces.
	void filltexture2(GLuint texture); //Specifies a global second texture indice to set on all faces.
	void normalizetexcoords(); //Makes the texcoord equal to the vertex X/Z position.
	void equalizetexcoords(); //Copies data from texcoord to texcoord2
	void modvertex(GLVECTOR3f vector);
	void mergebordernormals(Sector *right,Sector *bottom);
	void calculatenormals();
	void calculatefacematrices();
	


	//High-end functions: (Functions made to be used from outside)	
	void creategrid(float left,float top,float right,float bottom,float xstep = 1.0,float ystep = 1.0); //Physically creates the sector. Creates and initializes all needed objects to it's default settings.
	void setbump(float *map,int xsize,int ysize); //Sets (calculates) vertex height according to height map.
	void finalize(); //Call this after all modifications has been made after creating grid.	

	void render(GLenum mode = GL_TRIANGLES);

	Sector *clone(Sector *target = NULL); //Makes a perfect duplicate of this sector

	int intersecting(int index,GLVECTOR3f pos,GLVECTOR3f dir) {return(face[index].intersecting(pos-vertex[face[index].vertice[0]],dir));};

	
	//Debug functions:
	void rendernormals();
	void renderborder();
	int renderintersecting(GLVECTOR3f pos,GLVECTOR3f dir);
	

	//Internal / System functions:
	Sector();
	~Sector();
	void save(ofstream *fout,TextureHandler *th);
	void load(ifstream *fin,TextureHandler *th);
	void reporttextures(TextureHandler *th);



	void setfaceindex(); //(re)Calculates the most optimal face indice order to use. 
	void calcrendervertices();
	void setsectorarea(); //Finds out the boundary area of this sector.



//ENTITY FUNCTIONS
	Entity *entity[S_MAXENTITIES];
	int entities;

	void addentity(Entity *src);
	void deleteentity(Entity *src);
	void clearentities() {entities = 0;};

	void renderentities();
	void moveentities(DWORD tick);
};


void Sector::setcolor(int x,int y,GLVECTOR3f col,int action)
{
	int num,num2;
	int index = x+y*amx;
	for (num = 0; num < faces; num++)
	{
		for (num2 = 0; num2 < 3; num2++)
		{
			if (face[num].vertice[num2] == index)
			{
				switch(action)
				{
				case 0:
					face[num].color[num2] = col;
				break;
				case 1:
					face[num].color[num2] += col;
				break;
				case 2:
					face[num].color[num2] -= col;
				break;
				case 3:
					face[num].color[num2] *= col;
				break;
				}
			}
		}
	}
}

void Sector::setcolor2(int x,int y,GLVECTOR4f col,int action)
{
	int num,num2;
	int index = x+y*amx;
	for (num = 0; num < faces; num++)
	{
		for (num2 = 0; num2 < 3; num2++)
		{
			if (face[num].vertice[num2] == index)
			{
				switch(action)
				{
				case 0:
					face[num].color2[num2] = col;
				break;
				case 1:
					face[num].color2[num2] += col;
				break;
				case 2:
					face[num].color2[num2] -= col;
				break;
				case 3:
					face[num].color2[num2] *= col;
				break;
				}
			}
		}
	}
}







void Sector::renderentities()
{
	static int num;
	for (num = 0; num < entities; num++)
	{
		entity[num]->render();
	}

}
void Sector::moveentities(DWORD tick)
{
	static int num;
	for (num = 0; num < entities; num++)
	{
		entity[num]->move(tick);
	}

}

void Sector::addentity(Entity *src)
{
	static int num,num2;
	for (num = 0; num < entities; num++)
	{
		if (src->renderpriority <= entity[num]->renderpriority) break;
	}
	for (num2 = entities; num2 > num; num2--)
	{
		entity[num2] = entity[num2-1];
	}
	entity[num] = src;
	entities++;
}


void Sector::deleteentity(Entity *src)
{
	static int num,num2;
	for (num = 0; num < entities; num++)
	{
		if (entity[num] == src) 
		{
			entities--;
			for (num2 = num; num2 < entities; num2++)
			{
				entity[num2] = entity[num2+1];
			}
		}
	}
}



void Sector::renderborder()
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);
	glDisable(GL_TEXTURE_2D);

	glColor3f(0.3,0.3,0.3);
	glBegin(GL_QUADS);
		
		glVertex3f(getpos(SAREA_LEFT),getminheight(),getpos(SAREA_TOP));
		glVertex3f(getpos(SAREA_RIGHT),getminheight(),getpos(SAREA_TOP));
		glVertex3f(getpos(SAREA_RIGHT),getmaxheight(),getpos(SAREA_TOP));
		glVertex3f(getpos(SAREA_LEFT),getmaxheight(),getpos(SAREA_TOP));

		glVertex3f(getpos(SAREA_LEFT),getminheight(),getpos(SAREA_TOP));
		glVertex3f(getpos(SAREA_LEFT),getminheight(),getpos(SAREA_BOTTOM));
		glVertex3f(getpos(SAREA_LEFT),getmaxheight(),getpos(SAREA_BOTTOM));
		glVertex3f(getpos(SAREA_LEFT),getmaxheight(),getpos(SAREA_TOP));
/*
		glVertex3f(getpos(SAREA_LEFT),getminheight(),getpos(SAREA_BOTTOM));
		glVertex3f(getpos(SAREA_RIGHT),getminheight(),getpos(SAREA_BOTTOM));
		glVertex3f(getpos(SAREA_RIGHT),getmaxheight(),getpos(SAREA_BOTTOM));
		glVertex3f(getpos(SAREA_LEFT),getmaxheight(),getpos(SAREA_BOTTOM));

		glVertex3f(getpos(SAREA_RIGHT),getminheight(),getpos(SAREA_TOP));
		glVertex3f(getpos(SAREA_RIGHT),getminheight(),getpos(SAREA_BOTTOM));
		glVertex3f(getpos(SAREA_RIGHT),getmaxheight(),getpos(SAREA_BOTTOM));
		glVertex3f(getpos(SAREA_RIGHT),getmaxheight(),getpos(SAREA_TOP));
*/

	glEnd();

	glEnable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}


Sector *Sector::clone(Sector *target)
{
	if (!target)
	target = new Sector;

	memcpy(target,this,sizeof(Sector));

	target->vertex = new GLVECTOR3f[vertexes];
	memcpy(target->vertex,vertex,sizeof(GLVECTOR3f)*vertexes);

	target->normal = new GLVECTOR3f[vertexes];
	memcpy(target->normal,normal,sizeof(GLVECTOR3f)*vertexes);

//	target->color = new GLVECTOR3f[vertexes];
//	memcpy(target->color,color,sizeof(GLVECTOR3f)*vertexes);

//	target->color2 = new GLVECTOR4f[vertexes];
//	memcpy(target->color2,color2,sizeof(GLVECTOR4f)*vertexes);

	target->texcoord = new GLVECTOR2f[vertexes];
	memcpy(target->texcoord,texcoord,sizeof(GLVECTOR2f)*vertexes);

	target->texcoord2 = new GLVECTOR2f[vertexes];
	memcpy(target->texcoord2,texcoord2,sizeof(GLVECTOR2f)*vertexes);

	target->face = new SectorFace[faces];
	memcpy(target->face,face,sizeof(SectorFace)*faces);
	

	target->faceindex1 = new int[faces];
	memcpy(target->faceindex1,faceindex1,sizeof(int)*faces);

	target->faceindex2 = new int[faces];
	memcpy(target->faceindex2,faceindex2,sizeof(int)*faces);



	return(target);
}

void Sector::getcrossingfaces(GLVECTOR3f pos,GLVECTOR3f dir,int *indice,int max,int *count)
{
	*count = 0;
	int num;
	float len2;
	
	GLVECTOR2f l1,l2;
	GLVECTOR2f t1,t2;

	l1.set(pos.x,pos.z);
	l2.set(dir.x,dir.z);
	l2*=1000;
	l2+=l1;

	for (num = 0; num < faces; num++)
	{
		t1.set(vertex[face[num].vertice[0]].x,vertex[face[num].vertice[0]].z);
		t2.set(vertex[face[num].vertice[1]].x,vertex[face[num].vertice[1]].z);

		if (-1 != crossingline(&l1,&l2,&t1,&t2,&len2))
		if (len2 != 0 && len2 != 1)
		{
			
			indice[*count] = num;			
			*count += 1;
			if (*count >= max) return;
			continue;
		}
		t1.set(vertex[face[num].vertice[1]].x,vertex[face[num].vertice[1]].z);
		t2.set(vertex[face[num].vertice[2]].x,vertex[face[num].vertice[2]].z);

		if (-1 != crossingline(&l1,&l2,&t1,&t2,&len2))
		if (len2 != 0 && len2 != 1)
		{
			indice[*count] = num;			
			*count += 1;
			if (*count >= max) return;
			continue;
		}

	}

}


void Sector::calculatefacematrices()
{
	GLVECTOR3f v[3];
	int num;
	for (num = 0; num < faces; num++)
	{
		v[0] = vertex[face[num].vertice[1]] - vertex[face[num].vertice[0]];
		v[1] = vertex[face[num].vertice[2]] - vertex[face[num].vertice[0]];
		v[2] = calcnormal(v[0],v[1]);
		face[num].planematrix.setcolumn(0,v[0]);
		face[num].planematrix.setcolumn(1,v[1]);
		face[num].planematrix.setcolumn(2,v[2]);

		if (!face[num].planematrix.invert()) MessageBox(NULL,"Face index not invertable! (This should never happen)","",MB_OK);
	}

}



void Sector::mergebordernormals(Sector *right,Sector *bottom)
{
	int num;
	GLVECTOR3f temp;
	if (bottom)
	for (num = 0; num < amx; num++)
	{
		normal[(amy-1)*amx+num] = bottom->normal[num];
	}
	if (right)
	for (num = 0; num < amy; num++)
	{
		normal[(num+1)*amx-1] = right->normal[num*right->amx];
	}	
}


void Sector::reporttextures(TextureHandler *th)
{
	int num;
	for (num = 0; num < faces; num++)
	{
		if (face[num].texture != -1)
			th->report(face[num].texture);
		if (face[num].texture2 != -1)
			th->report(face[num].texture2);
	}
}


void Sector::load(ifstream *fin,TextureHandler *th)
{
	SectorFileHeader header;
	int num;



	fin->read((char*)&header,sizeof(header));

	vertexes = header.vertexes;
	amx = header.amx;
	amy = header.amy;

//	amx = (right-left+0.001)/xstep + 1;
//	amy = (bottom-top+0.001)/ystep + 1;

	creategrid(0,0,amx-1,amy-1);

	if (amx != header.amx || amy != header.amy) exit(0);

	faces = header.faces;
	xlen = header.xlen;
	ylen = header.ylen;

	fin->read((char*)vertex,sizeof(GLVECTOR3f)*vertexes);
	fin->read((char*)normal,sizeof(GLVECTOR3f)*vertexes);
//	fin->read((char*)color,sizeof(GLVECTOR3f)*vertexes);
//	fin->read((char*)color2,sizeof(GLVECTOR4f)*vertexes);
	fin->read((char*)texcoord,sizeof(GLVECTOR2f)*vertexes);
	fin->read((char*)texcoord2,sizeof(GLVECTOR2f)*vertexes);
	fin->read((char*)face,sizeof(SectorFace)*faces);


	for (num = 0; num < faces; num++)
	{
		face[num].texture = th->gettexture(face[num].texture);
		face[num].texture2 = th->gettexture(face[num].texture2);	
	}


}

void Sector::save(ofstream *fout,TextureHandler *th)
{
	int num;
	SectorFileHeader header;
	header.vertexes = vertexes;
	header.amx = amx;
	header.amy = amy;
	header.faces = faces;
	header.xlen = xlen;
	header.ylen = ylen;

	fout->write((char*)&header,sizeof(header));

	fout->write((char*)vertex,sizeof(GLVECTOR3f)*vertexes);
	fout->write((char*)normal,sizeof(GLVECTOR3f)*vertexes);
//	fout->write((char*)color,sizeof(GLVECTOR3f)*vertexes);
//	fout->write((char*)color2,sizeof(GLVECTOR4f)*vertexes);
	fout->write((char*)texcoord,sizeof(GLVECTOR2f)*vertexes);
	fout->write((char*)texcoord2,sizeof(GLVECTOR2f)*vertexes);

	for (num = 0; num < faces; num++)
	{
		face[num].texture = th->getindex(face[num].texture);
		face[num].texture2 = th->getindex(face[num].texture2);
	}

	fout->write((char*)face,sizeof(SectorFace)*faces);


	for (num = 0; num < faces; num++)
	{
		face[num].texture = th->gettexture(face[num].texture);
		face[num].texture2 = th->gettexture(face[num].texture2);	
	}


}







void Sector::finalize()
{
	setfaceindex(); 
	setsectorarea();
	calculatenormals();
	calculatefacematrices();
}

void Sector::rendernormals()
{
	
	static GLVECTOR3f temp;
	static int num; 
	glDisable(GL_TEXTURE_2D);
	glColor3f(1,1,1);
	glBegin(GL_LINES);
		for (num = 0; num < vertexes; num++)
		{
			glVertex3fv((float*)&vertex[num]);
			temp = vertex[num] + normal[num];
			glVertex3fv((float*)&temp);
		}

	glEnd();

	glEnable(GL_TEXTURE_2D);

}

void Sector::calculatenormals()
{
	static int xnum,ynum;
	static float k0,k1;
	for (ynum = 0; ynum < amy; ynum++)
		for (xnum = 0; xnum < amx; xnum++)
		{
			k0 = (vertex[xnum+(xnum +1 < amx)+ynum*amx].y - vertex[xnum-(xnum > 0)+ynum*amx].y)/2.0f;
			k1 = (vertex[xnum+(ynum+(ynum+1 < amy))*amx].y - vertex[xnum+(ynum-(ynum > 0))*amx].y)/2.0f;
			normal[xnum+ynum*amx].y = 1;
			normal[xnum+ynum*amx].x = -k0;
			normal[xnum+ynum*amx].z = -k1;
			normal[xnum+ynum*amx].normalize();
		}
}



float Sector::getheight(float x,float y)
{
	static float diastep;
	static int indexx,indexy;
	static float xhc,yhc;
	if (!inarea(x,y)) return(-999.9);
	x -= getpos(SAREA_LEFT);
	y -= getpos(SAREA_TOP);

	indexx = (int)(x/xlen);
	indexy = (int)(y/ylen);
	x -= indexx*xlen;
	y -= indexy*ylen;
	

	diastep = ylen/xlen;
	if (ylen-x*diastep >= y) //upper triangle
	{
		xhc = (vertex[indexx+indexy*amx+1].y - vertex[indexx+indexy*amx].y)/xlen;
		yhc = (vertex[indexx+(indexy+1)*amx].y - vertex[indexx+indexy*amx].y)/ylen;
		return(x*xhc+y*yhc+vertex[indexx+indexy*amx].y);

		
	}
	else //Lower triangle
	{
		
		xhc = (vertex[indexx+(indexy+1)*amx+1].y - vertex[indexx+(indexy+1)*amx].y)/xlen;
		yhc = (vertex[indexx+indexy*amx+1].y - vertex[indexx+(indexy+1)*amx+1].y)/ylen;
		return(x*xhc+(ylen-y)*yhc+vertex[indexx+(indexy+1)*amx].y);
	}



}

void Sector::setsectorarea()
{
	setmaxheight(vertex[0].y);
	setminheight(vertex[0].y);
	static float left;
	static float right;
	static float top;
	static float bottom;
	static int xnum;
	left = vertex[0].x;
	right = vertex[0].x;
	top = vertex[0].z;
	bottom = vertex[0].z;
	for (xnum = 0; xnum < vertexes; xnum++)
	{
		if (vertex[xnum].y > getmaxheight()) setmaxheight(vertex[xnum].y);
		if (vertex[xnum].y < getminheight()) setminheight(vertex[xnum].y);
		if (vertex[xnum].x < left) left = vertex[xnum].x;
		if (vertex[xnum].x > right) right = vertex[xnum].x;
		if (vertex[xnum].z < top) top = vertex[xnum].z;
		if (vertex[xnum].z > bottom) bottom = vertex[xnum].z;
	}

	setarea(left,top,right,bottom);

}

void Sector::modvertex(GLVECTOR3f vector)
{
	int num;
	for (num = 0; num < vertexes; num++)
	{
		vertex[num] *= vector;
	}
	setsectorarea();
	xlen *= vector.x;
	ylen *= vector.z;
}



void Sector::setfaceindex()
{
	int index;
	int fipos;
	int num;
	int num2;
	for (index = 0,fipos = 0; index < faces; index++,fipos++)
	{
		for (num = 0; num < fipos; num++)
		{
			if (face[index].texture <= face[faceindex1[num]].texture)
				break;
		}
		for (num2 = fipos; num2 > num; num2--)
			faceindex1[num2] = faceindex1[num2-1];

		faceindex1[num] = index;
	}

	for (index = 0,fipos = 0; index < faces; index++,fipos++)
	{
		for (num = 0; num < fipos; num++)
		{
			if (face[index].texture2 <= face[faceindex2[num]].texture2)
				break;
		}
		for (num2 = fipos; num2 > num; num2--)
			faceindex2[num2] = faceindex2[num2-1];

		faceindex2[num] = index;
	}
	
	
}

void Sector::calcbump(int x,int y,float *map,int mapx,int mapy,int mapxlen,int mapylen)
{
	int xstep = (float)(1.0f/xlen + 0.0001);
	int ystep = (float)(1.0f/ylen + 0.0001);
	float ret[1024];
	float leftbound[1024];
	float rightbound[1024];
	int xnum,ynum;
	float k0,k1;


	//render the left border
	k0 = (map[mapx+(mapy+1)*mapxlen] - map[mapx+(mapy-(mapy>0))*mapxlen])/2.0f;
	k1 = (map[mapx+(mapy+1+(mapy-2<mapylen))*mapxlen] - map[mapx+mapy*mapxlen])/2.0f;
	nurbline(map[mapx+mapy*mapxlen],map[mapx+(mapy+1)*mapxlen],k0,k1,ret,ystep);
	for (ynum = 0; ynum < ystep; ynum++)
		vertex[x+(y+ynum)*amx].y = ret[ynum];
	vertex[x+(y+ystep)*amx].y = map[mapx+(mapy+1)*mapxlen];


	//render the right border 
	mapx++;
	x += xstep;

	k0 = (map[mapx+(mapy+1)*mapxlen] - map[mapx+(mapy-(mapy>0))*mapxlen])/2.0f;
	k1 = (map[mapx+(mapy+1+(mapy-2<mapylen))*mapxlen] - map[mapx+mapy*mapxlen])/2.0f;
	nurbline(map[mapx+mapy*mapxlen],map[mapx+(mapy+1)*mapxlen],k0,k1,ret,ystep);
	for (ynum = 0; ynum < ystep; ynum++)
		vertex[x+(y+ynum)*amx].y = ret[ynum];
	vertex[x+(y+ystep)*amx].y = map[mapx+(mapy+1)*mapxlen];

	x -= xstep;
	mapx--;


	//render the left left side
	if (mapx - 1 >= 0)
	{
		k0 = (map[mapx-1+(mapy+1)*mapxlen]-map[mapx-1+(mapy-(mapy>0))*mapxlen])/2.0f;
		k1 = (map[mapx-1+(mapy+1+(mapy-2 < mapylen))*mapxlen] - map[mapx-1+mapy*mapxlen])/2.0f;

		nurbline(map[mapx-1+mapy*mapxlen],map[mapx-1+(mapy+1)*mapxlen],k0,k1,leftbound,ystep);
		leftbound[ystep] = map[mapx-1+(mapy+1)*mapxlen];


	}
	else
		for (ynum = 0; ynum <= ystep; ynum++)
		{
			leftbound[ynum] = vertex[x+(y+ynum)*amx].y;
		}

	//render the right right side
	if (mapx + 2 < mapxlen)
	{
		k0 = (map[mapx+2+(mapy+1)*mapxlen]-map[mapx+2+(mapy-(mapy>0))*mapxlen])/2.0f;
		k1 = (map[mapx+2+(mapy+1+(mapy-2 < mapylen))*mapxlen] - map[mapx+2+mapy*mapxlen])/2.0f;
		nurbline(map[mapx+2+mapy*mapxlen],map[mapx+2+(mapy+1)*mapxlen],k0,k1,rightbound,ystep);
		rightbound[ystep] = map[mapx+2+(mapy+1)*mapxlen];
	}
	else
		for (ynum = 0; ynum <= ystep; ynum++)
			rightbound[ynum] = vertex[x+xstep+(y+ynum)*amx].y;


	for (ynum = 0; ynum <= ystep; ynum++)
	{
		k0 = (vertex[x+xstep+(y+ynum)*amx].y - leftbound[ynum])/2.0f;
		k1 = (rightbound[ynum] - vertex[x+(y+ynum)*amx].y)/2.0f;
		nurbline(vertex[x+(y+ynum)*amx].y,vertex[x+xstep+(y+ynum)*amx].y,k0,k1,ret,xstep);
		for (xnum = 0; xnum < xstep; xnum++)
			vertex[x+xnum+(y+ynum)*amx].y = ret[xnum];
	}


}


void Sector::setbump(float *map,int xsize,int ysize)
{
	int xnum,ynum;
	int xstep = (float)(1.0f/xlen + 0.0001);
	int ystep = (float)(1.0f/ylen + 0.0001);
	int mapx,mapy;

/*
	//REMOVE AFTER DEBUGGING IS DONE!!!
	for (ynum = 0; ynum < amy; ynum++)
		for (xnum = 0; xnum < amx; xnum++)
//			vertex[xnum+ynum*amx].y = map[(int)vertex[xnum+ynum*amx].x+xsize*(int)vertex[xnum+ynum*amx].z];
			vertex[xnum+ynum*amx].y = -2;
*/

	for (ynum = 0; ynum < amy-1; ynum+=ystep)
		for (xnum = 0; xnum < amx-1; xnum+=xstep)
		{
			calcbump(xnum,ynum,map,vertex[0].x+xnum/xstep,vertex[0].z+ynum/ystep,xsize,ysize);
		}


	setsectorarea();

}


void Sector::equalizetexcoords()
{
	int num;
	for (num = 0; num < vertexes; num++)
	{
		texcoord2[num] = texcoord[num];
	}
}


void Sector::filltexture(GLuint texture)
{
	int num;
	for (num = 0; num < faces; num++)
	{
		face[num].texture = texture;
	}
}

void Sector::filltexture2(GLuint texture)
{
	int num;
	for (num = 0; num < faces; num++)
	{
		face[num].texture2 = texture;
	}
}


void Sector::fillcolor2(GLVECTOR4f col)
{
	int num;
	for (num = 0; num < faces; num++)
	{
		face[num].color2[0] = col;
		face[num].color2[1] = col;
		face[num].color2[2] = col;
	}
}


void Sector::fillcolor(GLVECTOR3f col)
{
	int num;
	for (num = 0; num < faces; num++)
	{
		face[num].color[0] = col;
		face[num].color[1] = col;
		face[num].color[2] = col;
	}
}


void Sector::normalizetexcoords()
{
	int num;
	int pos;
	for (num = 0; num < vertexes; num++)
	{
		texcoord[num].set(vertex[num].x,vertex[num].z);
	}
}


void Sector::render(GLenum mode)
{
	static int num;
	static GLuint curtex;
	static SectorFace *curface;
	curtex = -1;



	glDisable(GL_BLEND);
	glBegin(mode);
	for (num = 0; num < faces; num++)
	{
		curface = &face[faceindex1[num]];
		if (curface->texture != -1) //Remove after debug
		{
			if (curtex != curface->texture)
			{
				curtex = curface->texture;
				glEnd();
				glBindTexture(GL_TEXTURE_2D,curtex);
				glBegin(mode); 
			}
			glColor3fv((float*)&curface->color[0]);
			glTexCoord2fv((float*)&texcoord[curface->vertice[0]]);
			glNormal3fv((float*)&normal[curface->vertice[0]]);
			glVertex3fv((float*)&vertex[curface->vertice[0]]);

			glColor3fv((float*)&curface->color[1]);
			glTexCoord2fv((float*)&texcoord[curface->vertice[1]]);
			glNormal3fv((float*)&normal[curface->vertice[1]]);
			glVertex3fv((float*)&vertex[curface->vertice[1]]);

			glColor3fv((float*)&curface->color[2]);
			glTexCoord2fv((float*)&texcoord[curface->vertice[2]]);
			glNormal3fv((float*)&normal[curface->vertice[2]]);
			glVertex3fv((float*)&vertex[curface->vertice[2]]);
		}
	}

	glEnd();
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
//	glBlendFunc(GL_ZERO,GL_ZERO);

	glBegin(mode);
	for (num = 0; num < faces; num++)
	{
		curface = &face[faceindex2[num]];
		if (curface->texture2 != -1) 
		{
			if (curtex != curface->texture2)
			{
				curtex = curface->texture2;
				glEnd();
				glBindTexture(GL_TEXTURE_2D,curtex);
				glBegin(mode); //Is it needed to turn off/on for bindtexture?
			}
			glColor4fv((float*)&curface->color2[0]);
			glTexCoord2fv((float*)&texcoord2[curface->vertice[0]]);
			glNormal3fv((float*)&normal[curface->vertice[0]]);
			glVertex3fv((float*)&vertex[curface->vertice[0]]);

			glColor4fv((float*)&curface->color2[1]);
			glTexCoord2fv((float*)&texcoord2[curface->vertice[1]]);
			glNormal3fv((float*)&normal[curface->vertice[1]]);
			glVertex3fv((float*)&vertex[curface->vertice[1]]);

			glColor4fv((float*)&curface->color2[2]);
			glTexCoord2fv((float*)&texcoord2[curface->vertice[2]]);
			glNormal3fv((float*)&normal[curface->vertice[2]]);
			glVertex3fv((float*)&vertex[curface->vertice[2]]);
		}
	}
	glDisable(GL_BLEND);


	glEnd();
}






int Sector::renderintersecting(GLVECTOR3f pos,GLVECTOR3f dir)
{
	int count = 0;
	static int num;
	static GLuint curtex;
	static SectorFace *curface;
	curtex = -1;

	glColor3f(1,0,0);

	glDisable(GL_BLEND);
	glBegin(GL_TRIANGLES);

	int indice[1024];
	int indices;

	getcrossingfaces(pos,dir,indice,1024,&indices);


	for (num = 0; num < indices; num++)
	{
		curface = &face[indice[num]];
		if (intersecting(indice[num],pos,dir))
		{
			count++;
			if (curtex != curface->texture)
			{
				curtex = curface->texture;
				glEnd();
				glBindTexture(GL_TEXTURE_2D,curtex);
				glBegin(GL_TRIANGLES); 
			}
//			glColor3fv((float*)&color[curface->vertice[0]]);
			glTexCoord2fv((float*)&texcoord[curface->vertice[0]]);
			glNormal3fv((float*)&normal[curface->vertice[0]]);
			glVertex3fv((float*)&vertex[curface->vertice[0]]);

//			glColor3fv((float*)&color[curface->vertice[1]]);
			glTexCoord2fv((float*)&texcoord[curface->vertice[1]]);
			glNormal3fv((float*)&normal[curface->vertice[1]]);
			glVertex3fv((float*)&vertex[curface->vertice[1]]);

//			glColor3fv((float*)&color[curface->vertice[2]]);
			glTexCoord2fv((float*)&texcoord[curface->vertice[2]]);
			glNormal3fv((float*)&normal[curface->vertice[2]]);
			glVertex3fv((float*)&vertex[curface->vertice[2]]);
		}
	}

	glEnd();
/*	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
//	glBlendFunc(GL_ZERO,GL_ZERO);

	glBegin(mode);
	for (num = 0; num < faces; num++)
	{
		curface = &face[faceindex2[num]];
		if (curface->texture2 != -1) 
		{
			if (curtex != curface->texture2)
			{
				curtex = curface->texture2;
				glEnd();
				glBindTexture(GL_TEXTURE_2D,curtex);
				glBegin(mode); //Is it needed to turn off/on for bindtexture?
			}
			glColor4fv((float*)&color2[curface->vertice[0]]);
			glTexCoord2fv((float*)&texcoord2[curface->vertice[0]]);
			glNormal3fv((float*)&normal[curface->vertice[0]]);
			glVertex3fv((float*)&vertex[curface->vertice[0]]);

			glColor4fv((float*)&color2[curface->vertice[1]]);
			glTexCoord2fv((float*)&texcoord2[curface->vertice[1]]);
			glNormal3fv((float*)&normal[curface->vertice[1]]);
			glVertex3fv((float*)&vertex[curface->vertice[1]]);

			glColor4fv((float*)&color2[curface->vertice[2]]);
			glTexCoord2fv((float*)&texcoord2[curface->vertice[2]]);
			glNormal3fv((float*)&normal[curface->vertice[2]]);
			glVertex3fv((float*)&vertex[curface->vertice[2]]);
		}
	}



	glEnd();
*/
	return(count);
}






void Sector::clear()
{
	if (texcoord)
	{
		delete[] texcoord;
		texcoord = NULL;
	}
	if (texcoord2)
	{
		delete[] texcoord2;
		texcoord2 = NULL;
	}
	if (faceindex1)
	{
		delete[] faceindex1;
		faceindex1 = NULL;
	}
	if (faceindex2)
	{
		delete[] faceindex2;
		faceindex2 = NULL;
	}
	if (vertex)
	{
		delete[] vertex;
		vertex = NULL;
	}
/*	if (color)
	{
		delete[] color;
		color = NULL;
	}
	if (color2)
	{
		delete[] color2;
		color2 = NULL;
	}
*/
	if (normal)
	{
		delete[] normal;
		normal = NULL;
	}
	if (face)
	{
		delete[] face;
		face = NULL;
	}
}



Sector::~Sector()
{
	clear();
}

Sector::Sector()
{
	texcoord = NULL;
	texcoord2 = NULL;
	vertex = NULL;
	normal = NULL;
	face = NULL;
//	color = NULL;
//	color2 = NULL;
	faceindex1 = NULL;
	faceindex2 = NULL;
	clearentities();
}


void Sector::creategrid(float left,float top,float right,float bottom,float xstep,float ystep)
{

	amx = (right-left+0.001)/xstep + 1;
	amy = (bottom-top+0.001)/ystep + 1;
	xlen = xstep;
	ylen = ystep;

	clear();
	vertex = new GLVECTOR3f[amx*amy];
	vertexes = amx*amy;
	normal = new GLVECTOR3f[amx*amy];
//	color = new GLVECTOR3f[amx*amy];
//	color2 = new GLVECTOR4f[amx*amy];
	texcoord = new GLVECTOR2f[amx*amy];
	texcoord2 = new GLVECTOR2f[amx*amy];
	
	int xnum,ynum;
	for (ynum = 0; ynum < amy; ynum++)
	{
		for (xnum = 0; xnum < amx; xnum++)
		{
			vertex[xnum+ynum*amx].set(left+(float)xnum*xstep,0,top+(float)ynum*ystep);
			normal[xnum+ynum*amx].set(0,1,0);
//			color[xnum+ynum*amx].set(1,1,1);
//			color2[xnum+ynum*amx].set(1,1,1,0);
			texcoord[xnum+ynum*amx].set(vertex[xnum+ynum*amx].x,vertex[xnum+ynum*amx].z);
			texcoord2[xnum+ynum*amx] = texcoord[xnum+ynum*amx];
		}
	}

	setsectorarea();	

	amx--;
	amy--;


	xstep = 1/amx;
	ystep = 1/amy;

	faces = amx*amy*2;
	face = new SectorFace[faces];
	faceindex1 = new int[faces];
	faceindex2 = new int[faces];
	int index = 0;
	for (ynum = 0; ynum < amy; ynum++)
	{
		for (xnum = 0; xnum < amx; xnum++)
		{
			face[index].vertice[0] = xnum+ynum*(amy+1);
			face[index].vertice[1] = xnum+ynum*(amy+1)+1;
			face[index].vertice[2] = xnum+(ynum+1)*(amy+1);
			face[index].texture = -1;
			face[index].color[0].set(1,1,1);
			face[index].color[1].set(1,1,1);
			face[index].color[2].set(1,1,1);
			face[index].color2[0].set(1,1,1,0);
			face[index].color2[1].set(1,1,1,0);
			face[index].color2[2].set(1,1,1,0);
//			face[index].texcoord[0].set(xnum*xstep,ynum*ystep);
//			face[index].texcoord[1].set((xnum+1)*xstep,ynum*ystep);
//			face[index].texcoord[2].set(xnum*xstep,(ynum+1)*ystep);
			index++;

			face[index].vertice[0] = 1+xnum+(ynum+1)*(amy+1);
			face[index].vertice[1] = xnum+ynum*(amy+1)+1;
			face[index].vertice[2] = xnum+(ynum+1)*(amy+1);
			face[index].texture = -1;
//			face[index].texcoord[0].set((xnum+1)*xstep,(ynum+1)*ystep);
//			face[index].texcoord[1].set((xnum+1)*xstep,ynum*ystep);
//			face[index].texcoord[2].set(xnum*xstep,(ynum+1)*ystep);
			index++;
		}
	}

	for (xnum = 0; xnum < faces; xnum++) //Makes faceindexes in a poor, but yet valid order.
	{
		faceindex1[xnum] = xnum;
		faceindex2[xnum] = xnum;
	}

	amx++;
	amy++;

}




#endif