#ifndef __TEXTUREH__
#define __TECTUREH__

#define TH_TEXTURES 256
#define TH_FILE_BMP 0
#define TH_FILE_MPC 1

#include "mystuff/mpc.h"


struct TextureStruct
{
	char file[128];
	char filtertype;
	char filetype;
	GLuint texture;
};

class TextureHandler
{
public:
	TextureStruct texture[TH_TEXTURES];
	int textures;


	int saveindex[TH_TEXTURES];
	int saveindexes;
	void prepareforsave() {saveindexes = 0;};
	void report(GLuint tex);
	void finalizeforsave();
	void savetextures(ofstream *fout);
	void loadtextures(ifstream *fin);

	int exist(char *f);


	void clear() {textures = 0;};
	
	int loadbmp(char *file,char filtertype = 2);
	int load(char *file,char filtertype = 2);
	int add(GLuint newtexture);

	GLuint gettexture(int index) {if (index != -1) return(texture[index].texture); else return(-1);};
	int getindex(GLuint tex);
	char *getname(int index);

	TextureHandler *clone(TextureHandler *target = NULL);

	TextureHandler();

};


TextureHandler *TextureHandler::clone(TextureHandler *target)
{
	if (!target)
		target = new TextureHandler;

	memcpy(target,this,sizeof(TextureHandler));
	memcpy(target->texture,texture,sizeof(TextureStruct)*TH_TEXTURES);
	memcpy(target->saveindex,saveindex,sizeof(int)*TH_TEXTURES);
	

	return(target);
}


int TextureHandler::exist(char *f)
{
	int num;
	for (num = 0; num < textures; num++)
	{
		if (!strcmp(texture[num].file,f))
		{
			return(num);
		}
	}
	return(-1);
}

void TextureHandler::finalizeforsave()
{
	int num;
	TextureStruct temptexture[TH_TEXTURES];
	memcpy(temptexture,texture,sizeof(TextureStruct)*TH_TEXTURES);
	for (num = 0; num < saveindexes; num++)
	{
		memcpy(&texture[num],&temptexture[saveindex[num]],sizeof(TextureStruct));
//		texture[num] = temptexture[saveindex[num]];
	}
	textures = saveindexes;
}



void TextureHandler::savetextures(ofstream *fout)
{
	fout->write((char*)&textures,sizeof(int));
	fout->write((char*)texture,sizeof(TextureStruct)*textures);
}

void TextureHandler::loadtextures(ifstream *fin)
{
	int num;
	int end;


	clear();
	fin->read((char*)&textures,sizeof(int));
	fin->read((char*)texture,sizeof(TextureStruct)*textures);

	end = textures;
	textures = 0;

	for (num = 0; num < end; num++)
	{
		if (texture[num].filetype == TH_FILE_BMP)
			loadbmp(texture[num].file,texture[num].filtertype);
		else load(texture[num].file,texture[num].filtertype);
	}

}




void TextureHandler::report(GLuint tex)
{
	int num,num2;
	num = getindex(tex);
	if (num == -1) return;

	for (num2 = 0; num2 < saveindexes; num2++)
		if (saveindex[num2] == num)
			break;
	if (num2 == saveindexes) 
	{
		saveindex[saveindexes] = num;
		saveindexes++;
	}
}


int TextureHandler::getindex(GLuint tex)
{
	if (tex == -1) return(-1);
	int num;
	for (num = 0; num < textures; num++)
	{
		if (tex == texture[num].texture) return(num);
	}
	return(-1);
}


int TextureHandler::loadbmp(char *file,char filtertype)
{
	int ret;
	ret = exist(file);
	if (ret != -1) return(ret);

	Mpc mpc;
	mpc.loadbmp(file);
	char buf[128];
	strcpy(buf,file);
	ret = add(mpc.createtexture(filtertype));
	strcpy(texture[ret].file,buf);
	texture[ret].filtertype = filtertype;
	texture[ret].filetype = TH_FILE_BMP;
	return(ret);
}

int TextureHandler::load(char *file,char filtertype)
{
	int ret;
	ret = exist(file);
	if (ret != -1) return(ret);
	Mpc mpc;
	mpc.load(file);
	char buf[128];
	strcpy(buf,file);
	ret = add(mpc.createtexture(filtertype));
	strcpy(texture[ret].file,buf);
	texture[ret].filtertype = filtertype;
	texture[ret].filetype = TH_FILE_MPC;
	return(ret);
}


int TextureHandler::add(GLuint newtexture)
{
	texture[textures].texture = newtexture;
	texture[textures].file[0] = 0;
	texture[textures].filetype = -1;
	texture[textures].filtertype = 2;
	textures++;
	return(textures-1);
}



TextureHandler::TextureHandler()
{
	textures = 0;
}




#endif